#ifndef _TYPES_H_
#define _TYPES_H_

#ifndef AIX 
typedef unsigned char  uchar;
#endif 

#ifndef AIX 
typedef unsigned short ushort;
#endif 

#ifndef AIX 
typedef unsigned long  ulong;
#endif 

#ifndef AIX 
typedef unsigned int   uint;
#endif 

#endif
