/////////////////////////////////////////
// Converter.h
// simple calculator for min MUM length
/////////////////////////////////////////
// See the LICENSE file included with this software for license information.

#ifndef CONVERTER_H
#define CONVERTER_H


#include <string>
#include <math.h>
#include "Stack.h"

using namespace std;
int   Converter  ( string, string &, int );
float Calculator ( string, int, float);

#endif
